export interface PerformanceTestStats {
    min: number;
    sum: number;
    avg: number;
    med: number;
    med95: number;
    sd: number;
    repeats: number;
    gcTime?: number;
}
export interface PerformanceTestResultEntry {
    name: string;
    stats: PerformanceTestStats;
}
export declare type PerformanceTestResults = PerformanceTestResultEntry[];
export interface PerformanceTestSample {
    name: string;
    time: number;
}
export declare function calculateStats(samples: number[], gcTime?: number): PerformanceTestStats;
/**
 * Get current time in milliseconds.
 */
export declare const getCurrentTime: () => number;
/**
 * Measure time performance of code.
 *
 * Executes `test` code `repeats` times and measures: `min`, `med` (median), `sum` and `avg`
 * (average) execution times.
 *
 * [[performance.now]] is used as time provider, with fallback to `new Date().getTime()`.
 *
 * Measurement reports are saved for later and logged after all tests (if in Mocha environment). See
 * [[reportPerformanceAndReset]].
 *
 * Example:
 *
 *     it('performance test', async () => {
 *         await measurePerformanceSync("Array/grow", 50, () => {
 *             // the code under test
 *             // will be executed 50 times ----^^
 *         });
 *     });
 *
 * Will print report like this after all tests:
 *
 *     #performance: Array/grow: min=1 med=2 avg=1.8 sum=72 (50 repeats)
 *
 * @param name - name of performance test
 * @param repeats - number of test repeats
 * @param test - tested code
 */
export declare function measurePerformanceSync(name: string, repeats: number, test: () => void): Promise<void>;
/**
 * Measure throughput performance of code.
 *
 * Executes `test` code for `timeout` milliseconds and reports throughput and aggregated times:
 * `min`, `med` (median), `sum` and `avg` (average) execution times.
 *
 * [[performance.now]] is used as time provider, with fallback to `new Date().getTime()`.
 *
 * Measurement reports are saved for later and logged after all tests (if in Mocha environment). See
 * [[reportPerformanceAndReset]].
 *
 * Example:
 *
 *     it('throughput test', async () => {
 *         await measureThroughputSync("Array/grow", 100, () => {
 *             // the code under test
 *             // will be executed for 100 milliseconds ----^^
 *         });
 *     });
 *
 * Will print report like this after all tests:
 *
 *     #performance: Array/grow: min=1 med=2 avg=1.8 sum=72 repeats=123 throughput=1242/s
 *
 * @param name - name of performance test
 * @param repeats - number of test repeats
 * @param test - tested code
 */
export declare function measureThroughputSync(name: string, testDuration: number, test: () => void): Promise<void>;
export declare function addPerformanceResultsSample(name: string, time: number): void;
/**
 *
 */
export declare function reportPerformanceResults(results: PerformanceTestResults, baseline?: PerformanceTestResults): void;
export declare function reportPerformanceEntry(entry: PerformanceTestResultEntry): void;
export declare function reportPerformanceEntryWithBaseline(entry: PerformanceTestResultEntry, baseLineEntry: PerformanceTestResultEntry): void;
export declare function getPerformanceTestResults(): PerformanceTestResults;
/**
 * Report and reset performance measurement results.
 *
 * Designed to be called after round of tests. Shows results of all performance tests executed by
 * [[measurePerformanceSync]] and [[measureThroughputSync]].
 *
 * It resets results afterwards.
 *
 * If baseline is available (`.profile-helper-baseline.json`, customized by `PROFILEHELPER_OUTPUT`_)
 * then this run results are compared to baseline
 *
 * If `PROFILEHELPER_COMMAND=baseline` resutls are saved as baseline.
 *
 * In `Mocha` runtime it is called automatically in global `after` callback.
 */
export declare function reportPerformanceAndReset(): void;
/**
 * Report call.
 *
 * Convenience utility to be used temporarily in development to confirm expectations about number
 * of calls when measuring performance.
 *
 * Call counts are logged after all tests (if in Mocha environment). See
 * [[reportCallCountsAndReset]].
 *
 * Usage:
 *
 *     class Foo {
 *         push() {
 *             countCall("Foo#bar")
 *         }
 *     }
 *
 * It reports following after all tests:
 *
 *     #countCall: Foo#push called=123
 */
export declare function countCall(name: string, delta?: number): void;
/**
 * Count function/method calls decorator.
 *
 * Convenience utility to be used temporarily in development to confirm expectations about number
 * of calls when measuring performance.
 *
 * Call counts are logged after all tests (if in Mocha environment). See
 * [[reportCallCountsAndReset]].
 *
 * Usage - basic functional composition:
 *
 *     const bar = countCalls("bar",  () => { ... })
 *
 * Usage - experimental TypeScript decorators:
 *
 *     class Foo {
 *         @countCalls()
 *         push() {}
 *     }
 *
 * It reports following after execution of all tests:
 *
 *     ProfileHelper: Foo#push called=123
 *     ProfileHelper: bar called=1111
 */
export declare function countCalls<T extends (...args: any[]) => any>(name: string, fun?: T): T;
export declare function countCalls<T extends (...args: any[]) => any>(fun?: T): T;
export declare function countCalls(): MethodDecorator;
/**
 * Report and reset [[countCall]] results.
 *
 * Designed to be called after round of tests. Shows counters from all [[countCall]] calls.
 *
 * It resets results afterwards.
 *
 * In `Mocha` runtime it is called automatically in global `after` callback.
 */
export declare function reportCallCountsAndReset(): void;
//# sourceMappingURL=ProfileHelper.d.ts.map