import { LogLevel } from "@here/harp-utils";
import * as sinon from "sinon";
/**
 * Check if we're executing in Web Browser environment (but not in Web Worker)
 *
 * https://stackoverflow.com/questions/17575790/environment-detection-node-js-or-browser
 */
export declare function inBrowserContext(): boolean;
/**
 * Check if we're executing in Web Worker environment.
 */
export declare function inWebWorkerContext(): boolean;
/**
 * Define a suite that is executed only in Node.JS environment.
 */
export declare function inNodeContext(): boolean;
/**
 * Create stub of global constructor managed by sandbox.
 *
 * A `prototype` preserving, node/browser environment aware version of
 * `sandbox.stub(window | global, name).
 *
 * Use to stub global constructors like `Worker` or `XMLHttpRequest`.
 *
 * @param sandbox - `sinon.Sandbox` instance, required for proper cleanup after test
 * @param name - name of global symbol to be constructor
 */
export declare function stubGlobalConstructor(sandbox: sinon.SinonSandbox, name: string): sinon.SinonStub<any[], any> | sinon.SinonStub<unknown[], unknown>;
/**
 * Repeats block of code until it passes without `AssertionError`.
 *
 * Additionally, if test fails due to timeout, last error that was encountered is rethrown, so any
 * error that have constructor called `AssertionError` (matches chai assertions) will cause `test`
 * to be repeated after 1ms delay.
 *
 * The last error that blocked `willEventually` from resolving will be rethrown in `afterEach` to
 * mark which assertion didn't hold (see [[reportWillEventuallyBlockingAssertion]]).
 *
 * Use for APIs that are internally asynchronous without explicit means to monitor completion of
 * tasks.
 *
 * Example:
 *
 *   const foo = someCodeThatIsAsync({count: 6})
 *
 *   await willEventually(() => {
 *       assert.equals(foo.readyCount, 6);
 *   });
 *
 * @param test - closure with assertions that must pass
 * @returns promise that resolves when `test` passes without any error
 */
export declare function willEventually<T = void>(test: () => T): Promise<T>;
/**
 * Assert that promise is rejected.
 *
 * Check that promise `v` (passed directly or result of function) is eventually rejected with error
 * that matches `errorMessagePattern`.
 */
export declare function assertRejected(v: Promise<any> | (() => Promise<any>), errorMessagePattern: string | RegExp): Promise<void>;
declare type IConsoleLike = Pick<Console, "error" | "info" | "log" | "warn">;
/**
 * Assert that synchronous test function `fn` will logs specfic message.
 *
 * Captures error messages of `type` from `channel` and asserts that at least one contain
 * message that matches `errorMessagePattern`.
 *
 * Swallows matching messages, so they don't appear in actual output so "test log" is clean from
 * expected messages.
 *
 * Example with `console.warn`
 * ```
 *   assertWillLogSync(() => {
 *       console.warn("error: fooBar"),
 *       console,
 *       "warn"
 *       /error/
 *   );
 * ```
 *
 * @param fn - test function that shall provoke certain log output
 * @param channel - `Console` like object
 * @param type - type of console message i.e `"log" | "error" | "warn" | "info`
 * @param errorMessagePattern - string or regular expression that we look for in logs
 */
export declare function assertLogsSync(fn: () => void, channel: IConsoleLike, type: keyof IConsoleLike, errorMessagePattern: string | RegExp): void;
export interface EventSource<T> {
    addEventListener(type: string, listener: (event: T) => void): void;
    removeEventListener(type: string, listener: (event: T) => void): void;
}
/**
 * Wait for particular event on `THREE.EventDispatcher` or DOM `EventTarget` compatible objects.
 *
 * Automatically unregisters itself receiving the event.
 *
 * @param source - event source or target that has add/removeEventListener(type, listener) method.
 *     protocol
 * @param eventType - type of event
 * @returns promise that resolves to first event that is received
 */
export declare function waitForEvent<T>(source: EventSource<T>, eventType: string): Promise<T>;
/**
 * Sets the specified loggers to only log at the given minLogLevel. Only use this function when you
 * know that you can safely ignore a warning, otherwise you should consider to fix the issue. All
 * previous logging levels are reset after the function is executed.
 * @param loggerName The loggerName, or array of names to set to error
 * @param func The function to execute with the changed logging
 * @param minLogLevel The minimum log level that is shown, defaults to LogLevel.Error
 */
export declare function silenceLoggingAroundFunction(loggerName: string | string[], func: () => void, minLogLevel?: LogLevel): Promise<void>;
export {};
//# sourceMappingURL=TestUtils.d.ts.map