"use strict";
/*
 * Copyright (C) 2019-2021 HERE Europe B.V.
 * Licensed under Apache 2.0, see full license in LICENSE
 * SPDX-License-Identifier: Apache-2.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getWebGlInfo = exports.RenderingTestHelper = exports.setGlobalReporter = exports.getPlatform = void 0;
const harp_utils_1 = require("@here/harp-utils");
const chai_1 = require("chai");
const querystring = require("querystring");
const THREE = require("three");
const ua_parser_js_1 = require("ua-parser-js");
const DomImageUtils_1 = require("./DomImageUtils");
const DomReporter_1 = require("./DomReporter");
const ReferenceImageLocator_1 = require("./ReferenceImageLocator");
const RenderingTestResultReporter_1 = require("./RenderingTestResultReporter");
const logger = harp_utils_1.LoggerManager.instance.create("RenderingTestHelper");
/**
 * Defaults used for image comapision
 */
const TestOptionsDefaults = {
    /**
     * By default maxMismatchedPixels equal 0 to have pixel perfect match
     */
    maxMismatchedPixels: 0,
    /**
     * By default threshold equal 0 to have more sensitive comparision
     */
    threshold: 0
};
/**
 * Get `platform` property.
 *
 * Reads platform from:
 *  - query param `IBCT_PLATFORM_OVERRIDE` - overrides any detection
 *  - `navigator.userAgent` in form $name-$version-$os, optionally adds query param
 * `IBCT_PLATATFORM_EXTRA`
 *
 * `IBCT_PLATATFORM_EXTRA` is intended to inform about platform properties not detecatble from
 * browser api like "no-gpu" or "headless" or "-nvidia-gtx-whatever".
 */
function getPlatform() {
    let platformExtra = "";
    if (typeof window !== "undefined" && window.location) {
        if (window.location.search) {
            const queryParams = querystring.parse(window.location.search.substr(1));
            if (queryParams.IBCT_PLATFORM_OVERRIDE) {
                return queryParams.IBCT_PLATFORM_OVERRIDE;
            }
            if (queryParams.IBCT_PLATFORM_EXTRA) {
                platformExtra = queryParams.IBCT_PLATFORM_EXTRA;
            }
        }
    }
    const windowOverride = global.IBCT_PLATFORM_OVERRIDE;
    if (windowOverride !== undefined) {
        return windowOverride;
    }
    if (global.IBCT_PLATATFORM_EXTRA) {
        platformExtra = global.IBCT_PLATATFORM_EXTRA;
    }
    if (typeof navigator === "undefined") {
        return "nodejs";
    }
    else {
        // will have platforms like Chrome-69.2.1945-Linux
        const ua = new ua_parser_js_1.UAParser(navigator.userAgent);
        let platformParts = [ua.getBrowser().name, ua.getBrowser().version, ua.getOS().name];
        if (platformExtra) {
            platformParts.push(platformExtra);
        }
        platformParts = platformParts
            .filter(str => str !== undefined && str !== "")
            .map(str => str.replace(/ /g, ""));
        return platformParts.join("-");
    }
}
exports.getPlatform = getPlatform;
const domReporter = new DomReporter_1.DomReporter();
const feedbackServerReporter = new RenderingTestResultReporter_1.RenderingTestResultReporter("");
let ibctReporter = {
    reportImageComparisonResult() {
        const argsArray = Array.prototype.slice.call(arguments);
        domReporter.reportImageComparisonResult.apply(domReporter, argsArray);
        feedbackServerReporter.reportImageComparisonResult.apply(feedbackServerReporter, argsArray);
    }
};
function setGlobalReporter(reporter) {
    ibctReporter = reporter;
}
exports.setGlobalReporter = setGlobalReporter;
class RenderingTestHelper {
    constructor(mochaTest, baseImageProps) {
        this.mochaTest = mochaTest;
        this.baseImageProps = baseImageProps;
    }
    /**
     * Load image data using cache.
     */
    static cachedLoadImageData(url) {
        let p = this.preloadedImageCache.get(url);
        if (p !== undefined) {
            return p;
        }
        p = DomImageUtils_1.loadImageData(url);
        this.preloadedImageCache.set(url, p);
        return p;
    }
    /**
     * Save actual image only with comparison status OK
     *
     * @param canvas - actual image canvas
     * @param name - test name
     */
    async saveCanvasMatchesReference(canvas, name) {
        const actualImage = await DomImageUtils_1.canvasToImageData(canvas);
        const imageProps = Object.assign(Object.assign({}, this.baseImageProps), { platform: getPlatform(), name });
        const result = {
            mismatchedPixels: 0,
            diffImage: actualImage
        };
        ibctReporter.reportImageComparisonResult(imageProps, actualImage, true, actualImage, result);
    }
    /**
     * Compare actual image vs reference image then report comparison result to feedbackServer
     *
     * @param canvas - actual image canvas
     * @param name - test name
     * @param options - test options
     */
    async assertCanvasMatchesReference(canvas, name, options) {
        const actualImage = await DomImageUtils_1.canvasToImageData(canvas);
        const testOptions = Object.assign(Object.assign({}, TestOptionsDefaults), options);
        const imageProps = Object.assign(Object.assign({}, this.baseImageProps), { platform: getPlatform(), name });
        const referenceImageUrl = ReferenceImageLocator_1.getReferenceImageUrl(imageProps);
        let referenceImageData;
        try {
            referenceImageData = await RenderingTestHelper.cachedLoadImageData(referenceImageUrl);
        }
        catch (error) {
            logger.log(`[ERROR[ Reference image ${name} not found. Please update reference data`);
            ibctReporter.reportImageComparisonResult(imageProps, actualImage, false);
            this.mochaTest.skip();
            return;
        }
        const result = DomImageUtils_1.compareImages(actualImage, referenceImageData, testOptions);
        ibctReporter.reportImageComparisonResult(imageProps, actualImage, result.mismatchedPixels <= testOptions.maxMismatchedPixels, referenceImageData, result);
        chai_1.assert.equal(actualImage.height, referenceImageData.height);
        chai_1.assert.equal(actualImage.width, referenceImageData.width);
        chai_1.assert.isAtMost(result.mismatchedPixels, testOptions.maxMismatchedPixels, `${result.mismatchedPixels} mismatched pixels, reference image: ${name}`);
    }
}
exports.RenderingTestHelper = RenderingTestHelper;
RenderingTestHelper.preloadedImageCache = new Map();
function getWebGlInfo() {
    var _a;
    //Enable backward compatibility with three.js <= 0.117
    const renderer = new ((_a = THREE.WebGL1Renderer) !== null && _a !== void 0 ? _a : THREE.WebGLRenderer)();
    const context = renderer.getContext();
    const result = {};
    const availableExtensions = context.getSupportedExtensions();
    if (availableExtensions !== null &&
        availableExtensions.indexOf("WEBGL_debug_renderer_info") > -1) {
        const infoExtension = context.getExtension("WEBGL_debug_renderer_info");
        if (infoExtension !== null) {
            result.vendor = context.getParameter(infoExtension.UNMASKED_VENDOR_WEBGL);
            result.gpu = context.getParameter(infoExtension.UNMASKED_RENDERER_WEBGL);
        }
    }
    renderer.dispose();
    return result;
}
exports.getWebGlInfo = getWebGlInfo;
try {
    logger.log("WebGlInfo", getWebGlInfo());
}
catch (_a) {
    logger.warn("WebGL is not supported on this machine");
}
//# sourceMappingURL=RenderingTestHelper.js.map