"use strict";
/*
 * Copyright (C) 2019-2021 HERE Europe B.V.
 * Licensed under Apache 2.0, see full license in LICENSE
 * SPDX-License-Identifier: Apache-2.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
// @here:check-imports:environment:node
const harp_utils_1 = require("@here/harp-utils");
const program = require("commander");
const fs = require("fs");
const HtmlReport_1 = require("./HtmlReport");
const RenderingTestResultCommon_1 = require("./RenderingTestResultCommon");
const logger = harp_utils_1.LoggerManager.instance.create("RenderingTestResultCli");
let createMissingReferences = true;
let forceApprovals = false;
async function genConsoleReport(results) {
    let someTestsFailed = false;
    results.forEach(info => {
        const referencePath = RenderingTestResultCommon_1.getOutputImagePath(Object.assign(Object.assign({}, info.imageProps), { extra: ".reference" }), baseResultsPath);
        if (createMissingReferences && info.actualImagePath && !fs.existsSync(referencePath)) {
            logger.log(`${referencePath}: establishing reference image`);
            fs.copyFileSync(info.actualImagePath, referencePath);
        }
        else if (info.mismatchedPixels) {
            if ((info.approveDifference === true || forceApprovals) && info.actualImagePath) {
                logger.log(`${referencePath}: difference explicitly approved, establishing reference image`);
                fs.copyFileSync(info.actualImagePath, referencePath);
            }
            else {
                logger.log(`${referencePath}: failed, ${info.mismatchedPixels} mismatched pixels`);
                if (info.diffImagePath) {
                    logger.log(`   see ${info.diffImagePath}`);
                }
                someTestsFailed = true;
            }
        }
        else {
            logger.log(`${referencePath}: success`);
        }
    });
    return !someTestsFailed;
}
let baseResultsPath = "rendering-test-results";
async function main() {
    program.usage("[options] COMMAND").option("-D, --dir <DIR>", "base test results path", (value) => {
        baseResultsPath = value;
        return value;
    }, baseResultsPath);
    program
        .command("html-report")
        .description("generate HTML report from all IBCT test results")
        .alias("r")
        .action(async function () {
        const results = await RenderingTestResultCommon_1.loadSavedResults(baseResultsPath);
        const r = await HtmlReport_1.genHtmlReport(results, {}, baseResultsPath);
        const overallTestsResult = r[0];
        const reportHtml = r[1];
        if (process.stdout.isTTY) {
            const fileName = "ibct-report.html";
            logger.info(`(stdout is tty, saving HTML to ${fileName})`);
            fs.writeFileSync(fileName, reportHtml, "utf-8");
        }
        else {
            fs.writeSync(1, Buffer.from(reportHtml, "utf-8"));
        }
        process.exit(overallTestsResult ? 0 : 1);
    });
    program
        .command("approve")
        .description("approve current results locally")
        .alias("a")
        .action(async function () {
        logger.info("approving all actual images");
        const results = await RenderingTestResultCommon_1.loadSavedResults(baseResultsPath);
        forceApprovals = true;
        const overallTestsResult = await genConsoleReport(results);
        process.exit(overallTestsResult ? 0 : 1);
    });
    program
        .command("save-reference")
        .description("establish missing reference images")
        .alias("s")
        .action(async function () {
        logger.info("saving actual images as reference");
        const results = await RenderingTestResultCommon_1.loadSavedResults(baseResultsPath);
        createMissingReferences = true;
        const overallTestsResult = await genConsoleReport(results);
        process.exit(overallTestsResult ? 0 : 1);
    });
    return program.parse(process.argv);
}
main().catch(error => {
    logger.log("error:", error);
    process.exit(2);
});
//# sourceMappingURL=RenderingTestResultCli.js.map