"use strict";
/*
 * Copyright (C) 2019-2021 HERE Europe B.V.
 * Licensed under Apache 2.0, see full license in LICENSE
 * SPDX-License-Identifier: Apache-2.0
 */
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.startStandaloneServer = exports.expressMiddleware = exports.installMiddleware = exports.getReferenceImage = exports.getIbctReport = exports.postIbctFeedback = void 0;
// @here:check-imports:environment:node
const harp_utils_1 = require("@here/harp-utils");
const bodyParser = require("body-parser");
const express = require("express");
const fs = require("fs");
const mkpath = require("mkpath");
const path = require("path");
const serveStatic = require("serve-static");
const util = require("util");
const HtmlReport_1 = require("./HtmlReport");
const RenderingTestResultCommon_1 = require("./RenderingTestResultCommon");
const logger = harp_utils_1.LoggerManager.instance.create("RenderingTestResultServer");
const writeFile = util.promisify(fs.writeFile);
const promisedMkpath = util.promisify(mkpath);
let outputBasePath = "./rendering-test-results";
/**
 * Parse `base64`-encoded `data-uri` string and present result as tuple of:
 * * `contentType: string`
 * * `buffer: Buffer`.
 */
function parseDataUri(dataUri) {
    const split = dataUri.split(",");
    const metaRegex = /^data:(.+\/.+);(.*)$/;
    const matches = split[0].match(metaRegex);
    if (!matches || matches[2] !== "base64") {
        throw new Error("parseDataUri: invalid format of DataUri input");
    }
    return {
        contentType: matches[1],
        buffer: Buffer.from(split[1], "base64")
    };
}
let currentResults = [];
function updateCurrentResults(newResult) {
    const matchingIdx = currentResults.findIndex(baseResult => {
        return newResult.actualImagePath === baseResult.actualImagePath;
    });
    if (matchingIdx !== -1) {
        currentResults[matchingIdx] = newResult;
    }
    else {
        currentResults.push(newResult);
    }
}
async function postIbctFeedback(req, res) {
    try {
        const payload = req.body;
        logger.log(`received results for`, JSON.stringify(payload.imageProps));
        const evaluation = payload.comparisonResult === undefined
            ? "no reference image"
            : payload.comparisonResult.mismatchedPixels === 0
                ? "ok"
                : `failed: ${payload.comparisonResult.mismatchedPixels} wrong pixels`;
        logger.log(`/ibct-feedback: result: ${evaluation}`);
        const imageProps = payload.imageProps;
        const imageResultPath = RenderingTestResultCommon_1.getOutputImagePath(Object.assign(Object.assign({}, imageProps), { extra: ".ibct-result", extension: ".json" }), outputBasePath);
        const imageResult = {
            imageProps,
            passed: payload.passed
        };
        if (payload.actualImage) {
            const actualImage = parseDataUri(payload.actualImage);
            const actualImagePath = RenderingTestResultCommon_1.getOutputImagePath(Object.assign(Object.assign({}, imageProps), { extra: ".current" }), outputBasePath);
            logger.log(`writing current image: ${actualImagePath}`);
            await promisedMkpath(path.dirname(actualImagePath));
            await writeFile(actualImagePath, actualImage.buffer);
            imageResult.actualImagePath = actualImagePath;
        }
        if (payload.comparisonResult && payload.comparisonResult.diffImage) {
            const diffImage = parseDataUri(payload.comparisonResult.diffImage);
            const diffImagePath = RenderingTestResultCommon_1.getOutputImagePath(Object.assign(Object.assign({}, imageProps), { extra: ".diff" }), outputBasePath);
            logger.log(`writing diff image: ${diffImagePath}`);
            await promisedMkpath(path.dirname(diffImagePath));
            await writeFile(diffImagePath, diffImage.buffer);
            imageResult.diffImagePath = diffImagePath;
            imageResult.mismatchedPixels = payload.comparisonResult.mismatchedPixels;
        }
        logger.log(`writing report ${imageResultPath}`);
        await promisedMkpath(path.dirname(imageResultPath));
        await writeFile(imageResultPath, JSON.stringify(imageResult, null, 2));
        updateCurrentResults(imageResult);
        res.status(200).send("OK");
    }
    catch (error) {
        logger.error("error", error);
        res.status(500).send(`error: ${error}`);
    }
}
exports.postIbctFeedback = postIbctFeedback;
async function getIbctReport(req, res) {
    try {
        const [failed, report] = await HtmlReport_1.genHtmlReport(currentResults, {}, outputBasePath);
        logger.log("Tests failed: ", failed);
        res.status(200).contentType("text/html").send(report);
    }
    catch (error) {
        logger.error("error", error);
        res.status(500).send(`error: ${error}\n${error.stack}`);
    }
}
exports.getIbctReport = getIbctReport;
function getReferenceImage(req, res) {
    const imageProps = req.query;
    const relPath = RenderingTestResultCommon_1.getOutputImagePath(Object.assign(Object.assign({}, imageProps), { extra: ".reference" }), outputBasePath);
    res.sendFile(path.resolve(process.cwd(), relPath));
}
exports.getReferenceImage = getReferenceImage;
/**
 * Install RenderingTestResultServer in `express.Router`.
 *
 * Example usage i.e `webpack-dev-server` configuration in `webpack.config.js`:
 *
 *     devServer: {
 *       before: function(app) {
 *         require('ts-node/register'); // so we can load typescript seamlessly
 *         const RenderingTestResultServer = require(
 *          "coresdk/@here/harp-test-utils/lib/rendering/RenderingTestResultServer"
 *         );
 *         RenderingTestResultServer.installMiddleware(app);
 *       }
 *     }
 */
function installMiddleware(app, basePath) {
    RenderingTestResultCommon_1.loadSavedResults().then(results => {
        currentResults = results;
    });
    outputBasePath = basePath;
    // tslint gives a false alert b/c bodyParser is a function(deprecated) and a namespace.
    // We are using the non-deprecated namespace here, so all fine.
    const jsonParser = bodyParser.json({ limit: 1024 * 1024 * 16 });
    app.get("/ibct-report", jsonParser, getIbctReport);
    app.post("/ibct-feedback", jsonParser, postIbctFeedback);
    app.get("/reference-image", getReferenceImage);
    logger.info("serving IBCT report at /ibct-report endpoint");
    logger.info("accepting IBCT results at /ibct-feedback endpoint");
}
exports.installMiddleware = installMiddleware;
function createExpressRouter() {
    const router = express.Router();
    installMiddleware(router, outputBasePath);
    return router;
}
let expressRouter;
function expressMiddleware(req, res, next) {
    if (expressRouter === undefined) {
        expressRouter = createExpressRouter();
    }
    return expressRouter(req, res, next);
}
exports.expressMiddleware = expressMiddleware;
/**
 * Default export is `express` middleware function.
 *
 * Usage
 * ```
 * import RenderTestResultServer from "..."
 * app.use(RenderTestResultServer);
 * ```
 */
exports.default = expressMiddleware;
/**
 * Start `RenderingTestResultServer` as simple, standalone HTTP server which
 * * supports /ibct-feedback endpoint
 * * servers files from `process.cwd()`
 */
function startStandaloneServer(host, port) {
    const app = express();
    logger.log("start standaloneServer: ", outputBasePath);
    installMiddleware(app, outputBasePath);
    app.use(serveStatic(".", { index: ["index.html"] }));
    app.listen(port, host, () => {
        logger.log(`listening on port ${host}:${port}!`);
    });
}
exports.startStandaloneServer = startStandaloneServer;
/**
 * When ran as CLI.
 *
 * First argument is specified then use it as output file path where to store
 * files from running tests (actual results)
 * Default outputBasePath is "./ibct-results"
 */
if (require.main === module) {
    const host = (_a = process.env.HOST) !== null && _a !== void 0 ? _a : "localhost";
    const port = process.env.PORT ? parseInt(process.env.PORT, 10) : 8081;
    if (process.argv.length > 2) {
        outputBasePath = process.argv[2];
    }
    startStandaloneServer(host, port);
}
//# sourceMappingURL=RenderingTestResultServer.js.map